"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import p4.config.v1.p4types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class P4Info(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PKG_INFO_FIELD_NUMBER: builtins.int
    TABLES_FIELD_NUMBER: builtins.int
    ACTIONS_FIELD_NUMBER: builtins.int
    ACTION_PROFILES_FIELD_NUMBER: builtins.int
    COUNTERS_FIELD_NUMBER: builtins.int
    DIRECT_COUNTERS_FIELD_NUMBER: builtins.int
    METERS_FIELD_NUMBER: builtins.int
    DIRECT_METERS_FIELD_NUMBER: builtins.int
    CONTROLLER_PACKET_METADATA_FIELD_NUMBER: builtins.int
    VALUE_SETS_FIELD_NUMBER: builtins.int
    REGISTERS_FIELD_NUMBER: builtins.int
    DIGESTS_FIELD_NUMBER: builtins.int
    EXTERNS_FIELD_NUMBER: builtins.int
    TYPE_INFO_FIELD_NUMBER: builtins.int
    @property
    def pkg_info(self) -> global___PkgInfo: ...
    @property
    def tables(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Table]: ...
    @property
    def actions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Action]: ...
    @property
    def action_profiles(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ActionProfile]: ...
    @property
    def counters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Counter]: ...
    @property
    def direct_counters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DirectCounter]: ...
    @property
    def meters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Meter]: ...
    @property
    def direct_meters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DirectMeter]: ...
    @property
    def controller_packet_metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ControllerPacketMetadata]: ...
    @property
    def value_sets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValueSet]: ...
    @property
    def registers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Register]: ...
    @property
    def digests(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Digest]: ...
    @property
    def externs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Extern]: ...
    @property
    def type_info(self) -> p4.config.v1.p4types_pb2.P4TypeInfo: ...
    def __init__(self,
        *,
        pkg_info: typing.Optional[global___PkgInfo] = ...,
        tables: typing.Optional[typing.Iterable[global___Table]] = ...,
        actions: typing.Optional[typing.Iterable[global___Action]] = ...,
        action_profiles: typing.Optional[typing.Iterable[global___ActionProfile]] = ...,
        counters: typing.Optional[typing.Iterable[global___Counter]] = ...,
        direct_counters: typing.Optional[typing.Iterable[global___DirectCounter]] = ...,
        meters: typing.Optional[typing.Iterable[global___Meter]] = ...,
        direct_meters: typing.Optional[typing.Iterable[global___DirectMeter]] = ...,
        controller_packet_metadata: typing.Optional[typing.Iterable[global___ControllerPacketMetadata]] = ...,
        value_sets: typing.Optional[typing.Iterable[global___ValueSet]] = ...,
        registers: typing.Optional[typing.Iterable[global___Register]] = ...,
        digests: typing.Optional[typing.Iterable[global___Digest]] = ...,
        externs: typing.Optional[typing.Iterable[global___Extern]] = ...,
        type_info: typing.Optional[p4.config.v1.p4types_pb2.P4TypeInfo] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pkg_info",b"pkg_info","type_info",b"type_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action_profiles",b"action_profiles","actions",b"actions","controller_packet_metadata",b"controller_packet_metadata","counters",b"counters","digests",b"digests","direct_counters",b"direct_counters","direct_meters",b"direct_meters","externs",b"externs","meters",b"meters","pkg_info",b"pkg_info","registers",b"registers","tables",b"tables","type_info",b"type_info","value_sets",b"value_sets"]) -> None: ...
global___P4Info = P4Info

class Documentation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BRIEF_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    brief: typing.Text
    """A brief description of something, e.g. one sentence"""

    description: typing.Text
    """A more verbose description of something. Multiline is accepted. Markup
    format (if any) is TBD.
    """

    def __init__(self,
        *,
        brief: typing.Text = ...,
        description: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["brief",b"brief","description",b"description"]) -> None: ...
global___Documentation = Documentation

class PkgInfo(google.protobuf.message.Message):
    """Top-level package documentation describing the forwarding pipeline config
    Can be used to manage multiple P4 packages.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    DOC_FIELD_NUMBER: builtins.int
    ANNOTATIONS_FIELD_NUMBER: builtins.int
    ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
    ARCH_FIELD_NUMBER: builtins.int
    ORGANIZATION_FIELD_NUMBER: builtins.int
    CONTACT_FIELD_NUMBER: builtins.int
    URL_FIELD_NUMBER: builtins.int
    STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
    name: typing.Text
    """a definitive name for this configuration, e.g. switch.p4_v1.0"""

    version: typing.Text
    """configuration version, free-format string"""

    @property
    def doc(self) -> global___Documentation:
        """brief and detailed descriptions"""
        pass
    @property
    def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """Miscellaneous metadata, free-form; a way to extend PkgInfo"""
        pass
    @property
    def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
        """Optional. If present, the location of `annotations[i]` is given by
        `annotation_locations[i]`.
        """
        pass
    arch: typing.Text
    """the target architecture, e.g. "psa" """

    organization: typing.Text
    """organization which produced the configuration, e.g. "p4.org" """

    contact: typing.Text
    """contact info for support,e.g. "tech-support@acme.org" """

    url: typing.Text
    """url for more information, e.g.
    "http://support.p4.org/ref/p4/switch.p4_v1.0"
    """

    @property
    def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]:
        """Miscellaneous metadata, structured; a way to extend PkgInfo"""
        pass
    def __init__(self,
        *,
        name: typing.Text = ...,
        version: typing.Text = ...,
        doc: typing.Optional[global___Documentation] = ...,
        annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
        annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
        arch: typing.Text = ...,
        organization: typing.Text = ...,
        contact: typing.Text = ...,
        url: typing.Text = ...,
        structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["doc",b"doc"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["annotation_locations",b"annotation_locations","annotations",b"annotations","arch",b"arch","contact",b"contact","doc",b"doc","name",b"name","organization",b"organization","structured_annotations",b"structured_annotations","url",b"url","version",b"version"]) -> None: ...
global___PkgInfo = PkgInfo

class P4Ids(google.protobuf.message.Message):
    """wrapping the enum in a message to avoid name collisions in C++, where "enum
    values are siblings of their type, not children of it"
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Prefix:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _PrefixEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[P4Ids._Prefix.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: P4Ids._Prefix.ValueType  # 0
        ACTION: P4Ids._Prefix.ValueType  # 1
        """P4 language built-ins"""

        TABLE: P4Ids._Prefix.ValueType  # 2
        VALUE_SET: P4Ids._Prefix.ValueType  # 3
        CONTROLLER_HEADER: P4Ids._Prefix.ValueType  # 4
        PSA_EXTERNS_START: P4Ids._Prefix.ValueType  # 16
        """PSA externs"""

        ACTION_PROFILE: P4Ids._Prefix.ValueType  # 17
        COUNTER: P4Ids._Prefix.ValueType  # 18
        DIRECT_COUNTER: P4Ids._Prefix.ValueType  # 19
        METER: P4Ids._Prefix.ValueType  # 20
        DIRECT_METER: P4Ids._Prefix.ValueType  # 21
        REGISTER: P4Ids._Prefix.ValueType  # 22
        DIGEST: P4Ids._Prefix.ValueType  # 23
        OTHER_EXTERNS_START: P4Ids._Prefix.ValueType  # 128
        """externs for other architectures (vendor extensions)"""

        MAX: P4Ids._Prefix.ValueType  # 255
        """max value for an unsigned 8-bit byte
        requires protoc >= 3.5.0
        reserved 0x100 to max;
        """

    class Prefix(_Prefix, metaclass=_PrefixEnumTypeWrapper):
        """ids are allocated in such a way that it is possible based on an id to
        deduce the resource type (e.g. table, action, counter, ...). The
        most-significant byte of the 32-bit id encodes the resource type. The
        purpose of this enum is to define which value is used as the
        most-significant byte for each resource type. The P4 compiler must use
        these values when allocating ids for P4 objects. Other users of P4Info can
        refer to this enum to identify a resource type based on its id.
        """
        pass

    UNSPECIFIED: P4Ids.Prefix.ValueType  # 0
    ACTION: P4Ids.Prefix.ValueType  # 1
    """P4 language built-ins"""

    TABLE: P4Ids.Prefix.ValueType  # 2
    VALUE_SET: P4Ids.Prefix.ValueType  # 3
    CONTROLLER_HEADER: P4Ids.Prefix.ValueType  # 4
    PSA_EXTERNS_START: P4Ids.Prefix.ValueType  # 16
    """PSA externs"""

    ACTION_PROFILE: P4Ids.Prefix.ValueType  # 17
    COUNTER: P4Ids.Prefix.ValueType  # 18
    DIRECT_COUNTER: P4Ids.Prefix.ValueType  # 19
    METER: P4Ids.Prefix.ValueType  # 20
    DIRECT_METER: P4Ids.Prefix.ValueType  # 21
    REGISTER: P4Ids.Prefix.ValueType  # 22
    DIGEST: P4Ids.Prefix.ValueType  # 23
    OTHER_EXTERNS_START: P4Ids.Prefix.ValueType  # 128
    """externs for other architectures (vendor extensions)"""

    MAX: P4Ids.Prefix.ValueType  # 255
    """max value for an unsigned 8-bit byte
    requires protoc >= 3.5.0
    reserved 0x100 to max;
    """


    def __init__(self,
        ) -> None: ...
global___P4Ids = P4Ids

class Preamble(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    ALIAS_FIELD_NUMBER: builtins.int
    ANNOTATIONS_FIELD_NUMBER: builtins.int
    ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
    DOC_FIELD_NUMBER: builtins.int
    STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
    id: builtins.int
    """ids share the same number-space; e.g. table ids cannot overlap with counter
    ids. Even though this is irrelevant to this proto definition, the ids are
    allocated in such a way that it is possible based on an id to deduce the
    resource type (e.g. table, action, counter, ...). This means that code
    using these ids can detect if the wrong resource type is used
    somewhere. This also means that ids of different types can be mixed
    (e.g. direct resource list for a table) without ambiguity. Note that id 0
    is reserved and means "invalid id".
    """

    name: typing.Text
    """fully qualified name of the P4 object, e.g. c1.c2.ipv4_lpm"""

    alias: typing.Text
    """an alias (alternative name) for the P4 object, probably shorter than its
    fully qualified name. The only constraint is for it to be unique with
    respect to other P4 objects of the same type. By default, the compiler uses
    the shortest suffix of the name that uniquely identifies the object. For
    example if the P4 program contains two tables with names s.c1.t and s.c2.t,
    the default aliases will respectively be c1.t and c2.t. In the future, the
    P4 programmer may also be able to override the default alias for any P4
    object (TBD).
    """

    @property
    def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    @property
    def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
        """Optional. If present, the location of `annotations[i]` is given by
        `annotation_locations[i]`.
        """
        pass
    @property
    def doc(self) -> global___Documentation:
        """Documentation of the entity"""
        pass
    @property
    def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]: ...
    def __init__(self,
        *,
        id: builtins.int = ...,
        name: typing.Text = ...,
        alias: typing.Text = ...,
        annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
        annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
        doc: typing.Optional[global___Documentation] = ...,
        structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["doc",b"doc"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["alias",b"alias","annotation_locations",b"annotation_locations","annotations",b"annotations","doc",b"doc","id",b"id","name",b"name","structured_annotations",b"structured_annotations"]) -> None: ...
global___Preamble = Preamble

class Extern(google.protobuf.message.Message):
    """used to group all extern instances of the same type in one message"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXTERN_TYPE_ID_FIELD_NUMBER: builtins.int
    EXTERN_TYPE_NAME_FIELD_NUMBER: builtins.int
    INSTANCES_FIELD_NUMBER: builtins.int
    extern_type_id: builtins.int
    """the extern_type_id is unique for a given architecture and must be in the
    range [0x81, 0xfe].
    """

    extern_type_name: typing.Text
    @property
    def instances(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ExternInstance]: ...
    def __init__(self,
        *,
        extern_type_id: builtins.int = ...,
        extern_type_name: typing.Text = ...,
        instances: typing.Optional[typing.Iterable[global___ExternInstance]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["extern_type_id",b"extern_type_id","extern_type_name",b"extern_type_name","instances",b"instances"]) -> None: ...
global___Extern = Extern

class ExternInstance(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def info(self) -> google.protobuf.any_pb2.Any:
        """specific to the extern type, declared in a separate vendor-specific proto
        file
        """
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        info: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["info",b"info","preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["info",b"info","preamble",b"preamble"]) -> None: ...
global___ExternInstance = ExternInstance

class MatchField(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _MatchType:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _MatchTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MatchField._MatchType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: MatchField._MatchType.ValueType  # 0
        EXACT: MatchField._MatchType.ValueType  # 2
        LPM: MatchField._MatchType.ValueType  # 3
        TERNARY: MatchField._MatchType.ValueType  # 4
        RANGE: MatchField._MatchType.ValueType  # 5
        OPTIONAL: MatchField._MatchType.ValueType  # 6
    class MatchType(_MatchType, metaclass=_MatchTypeEnumTypeWrapper):
        pass

    UNSPECIFIED: MatchField.MatchType.ValueType  # 0
    EXACT: MatchField.MatchType.ValueType  # 2
    LPM: MatchField.MatchType.ValueType  # 3
    TERNARY: MatchField.MatchType.ValueType  # 4
    RANGE: MatchField.MatchType.ValueType  # 5
    OPTIONAL: MatchField.MatchType.ValueType  # 6

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    ANNOTATIONS_FIELD_NUMBER: builtins.int
    ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
    BITWIDTH_FIELD_NUMBER: builtins.int
    MATCH_TYPE_FIELD_NUMBER: builtins.int
    OTHER_MATCH_TYPE_FIELD_NUMBER: builtins.int
    DOC_FIELD_NUMBER: builtins.int
    TYPE_NAME_FIELD_NUMBER: builtins.int
    STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
    id: builtins.int
    name: typing.Text
    @property
    def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    @property
    def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
        """Optional. If present, the location of `annotations[i]` is given by
        `annotation_locations[i]`.
        """
        pass
    bitwidth: builtins.int
    match_type: global___MatchField.MatchType.ValueType
    other_match_type: typing.Text
    """used for architecture-specific match types which are not part of the core
    P4 language or of the PSA architecture.
    """

    @property
    def doc(self) -> global___Documentation:
        """Documentation of the match field"""
        pass
    @property
    def type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
        """unset if not user-defined type"""
        pass
    @property
    def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]: ...
    def __init__(self,
        *,
        id: builtins.int = ...,
        name: typing.Text = ...,
        annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
        annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
        bitwidth: builtins.int = ...,
        match_type: global___MatchField.MatchType.ValueType = ...,
        other_match_type: typing.Text = ...,
        doc: typing.Optional[global___Documentation] = ...,
        type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
        structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["doc",b"doc","match",b"match","match_type",b"match_type","other_match_type",b"other_match_type","type_name",b"type_name"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["annotation_locations",b"annotation_locations","annotations",b"annotations","bitwidth",b"bitwidth","doc",b"doc","id",b"id","match",b"match","match_type",b"match_type","name",b"name","other_match_type",b"other_match_type","structured_annotations",b"structured_annotations","type_name",b"type_name"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["match",b"match"]) -> typing.Optional[typing_extensions.Literal["match_type","other_match_type"]]: ...
global___MatchField = MatchField

class Table(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _IdleTimeoutBehavior:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _IdleTimeoutBehaviorEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Table._IdleTimeoutBehavior.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NO_TIMEOUT: Table._IdleTimeoutBehavior.ValueType  # 0
        NOTIFY_CONTROL: Table._IdleTimeoutBehavior.ValueType  # 1
    class IdleTimeoutBehavior(_IdleTimeoutBehavior, metaclass=_IdleTimeoutBehaviorEnumTypeWrapper):
        """this enum can be extended in the future with other behaviors, such as
        "HARD_EVICTION"
        """
        pass

    NO_TIMEOUT: Table.IdleTimeoutBehavior.ValueType  # 0
    NOTIFY_CONTROL: Table.IdleTimeoutBehavior.ValueType  # 1

    PREAMBLE_FIELD_NUMBER: builtins.int
    MATCH_FIELDS_FIELD_NUMBER: builtins.int
    ACTION_REFS_FIELD_NUMBER: builtins.int
    CONST_DEFAULT_ACTION_ID_FIELD_NUMBER: builtins.int
    IMPLEMENTATION_ID_FIELD_NUMBER: builtins.int
    DIRECT_RESOURCE_IDS_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    IDLE_TIMEOUT_BEHAVIOR_FIELD_NUMBER: builtins.int
    IS_CONST_TABLE_FIELD_NUMBER: builtins.int
    OTHER_PROPERTIES_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def match_fields(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MatchField]: ...
    @property
    def action_refs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ActionRef]:
        """even when the table is indirect (see implementation_id) below, this field
        includes all possible actions for the table; by using ActionRef instead of
        a repeated field of action ids, each action reference in a P4 table is able
        to have its own annotations
        """
        pass
    const_default_action_id: builtins.int
    """0 (default value) means that the table does not have a const default action"""

    implementation_id: builtins.int
    """P4 id of the "implementation" for this table (e.g. action profile id); 0
    (default value) means that the table is a regular (direct) match table. As
    of today, only action profiles are supported but other table
    implementations may be added in the future
    """

    @property
    def direct_resource_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """ids of the direct resources (if any) attached to this table; for now this
        includes only direct counters and direct meters, but other resources may be
        added in the future
        """
        pass
    size: builtins.int
    """max number of entries in table"""

    idle_timeout_behavior: global___Table.IdleTimeoutBehavior.ValueType
    """is idle timeout supported for this table?"""

    is_const_table: builtins.bool
    """table with static P4 entries, cannot be modified at runtime"""

    @property
    def other_properties(self) -> google.protobuf.any_pb2.Any:
        """architecture-specific table properties which are not part of the core P4
        language or of the PSA architecture.
        """
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        match_fields: typing.Optional[typing.Iterable[global___MatchField]] = ...,
        action_refs: typing.Optional[typing.Iterable[global___ActionRef]] = ...,
        const_default_action_id: builtins.int = ...,
        implementation_id: builtins.int = ...,
        direct_resource_ids: typing.Optional[typing.Iterable[builtins.int]] = ...,
        size: builtins.int = ...,
        idle_timeout_behavior: global___Table.IdleTimeoutBehavior.ValueType = ...,
        is_const_table: builtins.bool = ...,
        other_properties: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["other_properties",b"other_properties","preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action_refs",b"action_refs","const_default_action_id",b"const_default_action_id","direct_resource_ids",b"direct_resource_ids","idle_timeout_behavior",b"idle_timeout_behavior","implementation_id",b"implementation_id","is_const_table",b"is_const_table","match_fields",b"match_fields","other_properties",b"other_properties","preamble",b"preamble","size",b"size"]) -> None: ...
global___Table = Table

class ActionRef(google.protobuf.message.Message):
    """used to list all possible actions in a Table"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Scope:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ScopeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionRef._Scope.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        TABLE_AND_DEFAULT: ActionRef._Scope.ValueType  # 0
        TABLE_ONLY: ActionRef._Scope.ValueType  # 1
        DEFAULT_ONLY: ActionRef._Scope.ValueType  # 2
    class Scope(_Scope, metaclass=_ScopeEnumTypeWrapper):
        pass

    TABLE_AND_DEFAULT: ActionRef.Scope.ValueType  # 0
    TABLE_ONLY: ActionRef.Scope.ValueType  # 1
    DEFAULT_ONLY: ActionRef.Scope.ValueType  # 2

    ID_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    ANNOTATIONS_FIELD_NUMBER: builtins.int
    ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
    STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
    id: builtins.int
    scope: global___ActionRef.Scope.ValueType
    @property
    def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    @property
    def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
        """Optional. If present, the location of `annotations[i]` is given by
        `annotation_locations[i]`.
        """
        pass
    @property
    def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]: ...
    def __init__(self,
        *,
        id: builtins.int = ...,
        scope: global___ActionRef.Scope.ValueType = ...,
        annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
        annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
        structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["annotation_locations",b"annotation_locations","annotations",b"annotations","id",b"id","scope",b"scope","structured_annotations",b"structured_annotations"]) -> None: ...
global___ActionRef = ActionRef

class Action(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Param(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ID_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        ANNOTATIONS_FIELD_NUMBER: builtins.int
        ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
        BITWIDTH_FIELD_NUMBER: builtins.int
        DOC_FIELD_NUMBER: builtins.int
        TYPE_NAME_FIELD_NUMBER: builtins.int
        STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
        id: builtins.int
        name: typing.Text
        @property
        def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
        @property
        def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
            """Optional. If present, the location of `annotations[i]` is given by
            `annotation_locations[i]`.
            """
            pass
        bitwidth: builtins.int
        @property
        def doc(self) -> global___Documentation:
            """Documentation of the Param"""
            pass
        @property
        def type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
            """unset if not user-defined type"""
            pass
        @property
        def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]: ...
        def __init__(self,
            *,
            id: builtins.int = ...,
            name: typing.Text = ...,
            annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
            annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
            bitwidth: builtins.int = ...,
            doc: typing.Optional[global___Documentation] = ...,
            type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
            structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["doc",b"doc","type_name",b"type_name"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["annotation_locations",b"annotation_locations","annotations",b"annotations","bitwidth",b"bitwidth","doc",b"doc","id",b"id","name",b"name","structured_annotations",b"structured_annotations","type_name",b"type_name"]) -> None: ...

    PREAMBLE_FIELD_NUMBER: builtins.int
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def params(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Action.Param]: ...
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        params: typing.Optional[typing.Iterable[global___Action.Param]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params","preamble",b"preamble"]) -> None: ...
global___Action = Action

class ActionProfile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _SelectorSizeSemantics:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _SelectorSizeSemanticsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ActionProfile._SelectorSizeSemantics.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SUM_OF_WEIGHTS: ActionProfile._SelectorSizeSemantics.ValueType  # 0
        """indicates that `size` and `max_group_size` represent the maximum sum of
        weights that can be present across all selector groups and within a
        single selector group respectively.
        """

        SUM_OF_MEMBERS: ActionProfile._SelectorSizeSemantics.ValueType  # 1
        """indicates that `size` and `max_group_size` represent the maximum number
        of members that can be present across all selector groups and within a
        single selector group respectively.
        """

    class SelectorSizeSemantics(_SelectorSizeSemantics, metaclass=_SelectorSizeSemanticsEnumTypeWrapper):
        pass

    SUM_OF_WEIGHTS: ActionProfile.SelectorSizeSemantics.ValueType  # 0
    """indicates that `size` and `max_group_size` represent the maximum sum of
    weights that can be present across all selector groups and within a
    single selector group respectively.
    """

    SUM_OF_MEMBERS: ActionProfile.SelectorSizeSemantics.ValueType  # 1
    """indicates that `size` and `max_group_size` represent the maximum number
    of members that can be present across all selector groups and within a
    single selector group respectively.
    """


    PREAMBLE_FIELD_NUMBER: builtins.int
    TABLE_IDS_FIELD_NUMBER: builtins.int
    WITH_SELECTOR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    MAX_GROUP_SIZE_FIELD_NUMBER: builtins.int
    SELECTOR_SIZE_SEMANTICS_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def table_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """the ids of the tables sharing this action profile"""
        pass
    with_selector: builtins.bool
    """true iff the action profile used dynamic selection"""

    size: builtins.int
    """max number of member entries across all groups if the action profile does
    not have a selector. Otherwise, semantics as specified by
    `selector_size_semantics` below.
    """

    max_group_size: builtins.int
    """0 if the action profile does not have a selector. Otherwise, semantics as
    specified by `selector_size_semantics` below.
    """

    selector_size_semantics: global___ActionProfile.SelectorSizeSemantics.ValueType
    """specifies the semantics of `size` and `max_group_size` above"""

    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        table_ids: typing.Optional[typing.Iterable[builtins.int]] = ...,
        with_selector: builtins.bool = ...,
        size: builtins.int = ...,
        max_group_size: builtins.int = ...,
        selector_size_semantics: global___ActionProfile.SelectorSizeSemantics.ValueType = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_group_size",b"max_group_size","preamble",b"preamble","selector_size_semantics",b"selector_size_semantics","size",b"size","table_ids",b"table_ids","with_selector",b"with_selector"]) -> None: ...
global___ActionProfile = ActionProfile

class CounterSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Unit:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _UnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CounterSpec._Unit.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: CounterSpec._Unit.ValueType  # 0
        BYTES: CounterSpec._Unit.ValueType  # 1
        PACKETS: CounterSpec._Unit.ValueType  # 2
        BOTH: CounterSpec._Unit.ValueType  # 3
    class Unit(_Unit, metaclass=_UnitEnumTypeWrapper):
        """Corresponds to 'type' constructor parameter for Counter / DirectCounter in
        PSA
        """
        pass

    UNSPECIFIED: CounterSpec.Unit.ValueType  # 0
    BYTES: CounterSpec.Unit.ValueType  # 1
    PACKETS: CounterSpec.Unit.ValueType  # 2
    BOTH: CounterSpec.Unit.ValueType  # 3

    UNIT_FIELD_NUMBER: builtins.int
    unit: global___CounterSpec.Unit.ValueType
    def __init__(self,
        *,
        unit: global___CounterSpec.Unit.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["unit",b"unit"]) -> None: ...
global___CounterSpec = CounterSpec

class Counter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    INDEX_TYPE_NAME_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def spec(self) -> global___CounterSpec: ...
    size: builtins.int
    """number of entries in the counter array"""

    @property
    def index_type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
        """unset if index is not user-defined type"""
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        spec: typing.Optional[global___CounterSpec] = ...,
        size: builtins.int = ...,
        index_type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","size",b"size","spec",b"spec"]) -> None: ...
global___Counter = Counter

class DirectCounter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    DIRECT_TABLE_ID_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def spec(self) -> global___CounterSpec: ...
    direct_table_id: builtins.int
    """the id of the table to which the counter is attached"""

    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        spec: typing.Optional[global___CounterSpec] = ...,
        direct_table_id: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble","spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["direct_table_id",b"direct_table_id","preamble",b"preamble","spec",b"spec"]) -> None: ...
global___DirectCounter = DirectCounter

class MeterSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Unit:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _UnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MeterSpec._Unit.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: MeterSpec._Unit.ValueType  # 0
        BYTES: MeterSpec._Unit.ValueType  # 1
        PACKETS: MeterSpec._Unit.ValueType  # 2
    class Unit(_Unit, metaclass=_UnitEnumTypeWrapper):
        """Corresponds to 'type' constructor parameter for Meter / DirectMeter in PSA"""
        pass

    UNSPECIFIED: MeterSpec.Unit.ValueType  # 0
    BYTES: MeterSpec.Unit.ValueType  # 1
    PACKETS: MeterSpec.Unit.ValueType  # 2

    UNIT_FIELD_NUMBER: builtins.int
    unit: global___MeterSpec.Unit.ValueType
    def __init__(self,
        *,
        unit: global___MeterSpec.Unit.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["unit",b"unit"]) -> None: ...
global___MeterSpec = MeterSpec

class Meter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    INDEX_TYPE_NAME_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def spec(self) -> global___MeterSpec: ...
    size: builtins.int
    """number of entries in the meter array"""

    @property
    def index_type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
        """unset if index is not user-defined type"""
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        spec: typing.Optional[global___MeterSpec] = ...,
        size: builtins.int = ...,
        index_type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","size",b"size","spec",b"spec"]) -> None: ...
global___Meter = Meter

class DirectMeter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    DIRECT_TABLE_ID_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def spec(self) -> global___MeterSpec: ...
    direct_table_id: builtins.int
    """the id of the table to which the meter is attached"""

    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        spec: typing.Optional[global___MeterSpec] = ...,
        direct_table_id: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble","spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["direct_table_id",b"direct_table_id","preamble",b"preamble","spec",b"spec"]) -> None: ...
global___DirectMeter = DirectMeter

class ControllerPacketMetadata(google.protobuf.message.Message):
    """Any metadata associated with controller Packet-IO (Packet-In or Packet-Out)
    is modeled as P4 headers carrying special annotations
    @controller_header("packet_out") and @controller_header("packet_in")
    respectively. There can be at most one header each with these annotations.
    This message captures the info contained within these special headers,
    and used in p4runtime.proto to supply the metadata.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Metadata(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ID_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        ANNOTATIONS_FIELD_NUMBER: builtins.int
        ANNOTATION_LOCATIONS_FIELD_NUMBER: builtins.int
        BITWIDTH_FIELD_NUMBER: builtins.int
        TYPE_NAME_FIELD_NUMBER: builtins.int
        STRUCTURED_ANNOTATIONS_FIELD_NUMBER: builtins.int
        id: builtins.int
        name: typing.Text
        """This is the name of the header field (not fully-qualified), similar
        to e.g. Action.Param.name.
        """

        @property
        def annotations(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
        @property
        def annotation_locations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.SourceLocation]:
            """Optional. If present, the location of `annotations[i]` is given by
            `annotation_locations[i]`.
            """
            pass
        bitwidth: builtins.int
        @property
        def type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
            """unset if not user-defined type"""
            pass
        @property
        def structured_annotations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[p4.config.v1.p4types_pb2.StructuredAnnotation]: ...
        def __init__(self,
            *,
            id: builtins.int = ...,
            name: typing.Text = ...,
            annotations: typing.Optional[typing.Iterable[typing.Text]] = ...,
            annotation_locations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.SourceLocation]] = ...,
            bitwidth: builtins.int = ...,
            type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
            structured_annotations: typing.Optional[typing.Iterable[p4.config.v1.p4types_pb2.StructuredAnnotation]] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["type_name",b"type_name"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["annotation_locations",b"annotation_locations","annotations",b"annotations","bitwidth",b"bitwidth","id",b"id","name",b"name","structured_annotations",b"structured_annotations","type_name",b"type_name"]) -> None: ...

    PREAMBLE_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble:
        """preamble.name and preamble.id will specify header type ("packet_out" or
        "packet_in" for now).
        """
        pass
    @property
    def metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ControllerPacketMetadata.Metadata]:
        """Ordered based on header layout.
        This is a constraint on the generator of this P4Info.
        """
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        metadata: typing.Optional[typing.Iterable[global___ControllerPacketMetadata.Metadata]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["metadata",b"metadata","preamble",b"preamble"]) -> None: ...
global___ControllerPacketMetadata = ControllerPacketMetadata

class ValueSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    MATCH_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def match(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MatchField]: ...
    size: builtins.int
    """number of entries in the value_set, as per the P4 constructor call."""

    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        match: typing.Optional[typing.Iterable[global___MatchField]] = ...,
        size: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["match",b"match","preamble",b"preamble","size",b"size"]) -> None: ...
global___ValueSet = ValueSet

class Register(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    TYPE_SPEC_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    INDEX_TYPE_NAME_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def type_spec(self) -> p4.config.v1.p4types_pb2.P4DataTypeSpec: ...
    size: builtins.int
    @property
    def index_type_name(self) -> p4.config.v1.p4types_pb2.P4NamedType:
        """unset if index is not user-defined type"""
        pass
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        type_spec: typing.Optional[p4.config.v1.p4types_pb2.P4DataTypeSpec] = ...,
        size: builtins.int = ...,
        index_type_name: typing.Optional[p4.config.v1.p4types_pb2.P4NamedType] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","type_spec",b"type_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["index_type_name",b"index_type_name","preamble",b"preamble","size",b"size","type_spec",b"type_spec"]) -> None: ...
global___Register = Register

class Digest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREAMBLE_FIELD_NUMBER: builtins.int
    TYPE_SPEC_FIELD_NUMBER: builtins.int
    @property
    def preamble(self) -> global___Preamble: ...
    @property
    def type_spec(self) -> p4.config.v1.p4types_pb2.P4DataTypeSpec: ...
    def __init__(self,
        *,
        preamble: typing.Optional[global___Preamble] = ...,
        type_spec: typing.Optional[p4.config.v1.p4types_pb2.P4DataTypeSpec] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["preamble",b"preamble","type_spec",b"type_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["preamble",b"preamble","type_spec",b"type_spec"]) -> None: ...
global___Digest = Digest



"P4 Value conversion functions."

# Copyright (c) 2022 Bill Fisher
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
from typing import SupportsInt

from macaddress import MAC as MACAddress


class DecodeHint(enum.Enum):
    "Indicate how to decode values."

    DEFAULT = enum.auto()
    ADDRESS = enum.auto()


_ExactValue = SupportsInt | str | bytes
_ExactReturn = int | IPv4Address | IPv6Address | MACAddress

_LPMValue = str | IPv4Network | IPv6Network | tuple[_ExactValue, int]
_LPMReturn = tuple[_ExactReturn, int]

_TernaryValue = str | IPv4Network | IPv6Network | tuple[_ExactValue, _ExactValue]
_TernaryReturn = tuple[_ExactReturn, _ExactReturn]

_RangeValue = str | tuple[_ExactValue, _ExactValue]
_RangeReturn = tuple[_ExactReturn, _ExactReturn]

P4ParamValue = _ExactValue


def p4r_minimum_string_size(bitwidth: int) -> int:
    "P4Runtime `minimum_string_size` function (P4R-Spec section 8.4)"

    if bitwidth <= 0:
        raise ValueError(f"invalid bitwidth: {bitwidth}")
    return (bitwidth + 7) // 8


def p4r_truncate(value: bytes, signed: bool = False) -> bytes:
    "Truncate a bytes value to the specified bitwidth."
    assert not signed, "TODO: signed not yet supported"

    result = value.lstrip(b"\x00")
    return result if result else b"\x00"


def _parse_str(value: str, bitwidth: int) -> int:
    "Convert string value to an integer."
    value = value.strip()

    if bitwidth == 32 and "." in value:
        return int(IPv4Address(value))

    if bitwidth == 128 and ":" in value:
        return int(IPv6Address(value))

    if bitwidth == 48 and ("-" in value[1:] or ":" in value):
        return int(MACAddress(value))

    try:
        return int(value, base=0)
    except ValueError:
        raise ValueError(f"invalid value for bitwidth {bitwidth}: {value!r}") from None


def _parse_int(value: SupportsInt) -> int:
    "Convert SupportsInt value to an integer."
    if isinstance(value, float) and not value.is_integer():
        raise ValueError(f"fractional float value: {value!r}")

    # Handle int, non-fractional float and IP address objects.
    return int(value)


def encode_exact(value: _ExactValue, bitwidth: int) -> bytes:
    "Encode an exact field value into a byte encoding."
    assert value is not None

    if isinstance(value, bytes):
        return p4r_truncate(value)

    if isinstance(value, str):
        ival = _parse_str(value, bitwidth)
    elif isinstance(
        value, SupportsInt
    ):  # pyright: ignore [reportUnnecessaryIsInstance]
        ival = _parse_int(value)
    else:
        raise ValueError(f"invalid value type: {value!r}")

    if ival >= (1 << bitwidth):
        raise OverflowError(f"invalid value for bitwidth {bitwidth}: {value!r}")

    size = p4r_minimum_string_size(bitwidth)
    bval = ival.to_bytes(size, "big")
    return p4r_truncate(bval)


def decode_exact(
    data: bytes,
    bitwidth: int,
    hint: DecodeHint = DecodeHint.DEFAULT,
) -> _ExactReturn:
    """Decode a P4R value into an integer or address."""
    if not data:
        raise ValueError("empty data")

    ival = int.from_bytes(data, "big")
    if ival >= (1 << bitwidth):
        raise OverflowError(f"invalid value for bitwidth {bitwidth}: {data!r}")

    match hint:
        case DecodeHint.DEFAULT:
            return ival
        case DecodeHint.ADDRESS if bitwidth == 128:
            return IPv6Address(ival)
        case DecodeHint.ADDRESS if bitwidth == 48:
            return MACAddress(ival)
        case DecodeHint.ADDRESS if bitwidth == 32:
            return IPv4Address(ival)
        case DecodeHint.ADDRESS:
            return ival

    raise ValueError(f"invalid hint: {hint!r}")


def encode_lpm(value: _LPMValue, bitwidth: int) -> tuple[bytes, int]:
    "Encode a string value into a P4R LPM value."
    assert value is not None

    if isinstance(value, (IPv4Network, IPv6Network)):
        if bitwidth != value.max_prefixlen:
            raise ValueError(f"invalid value for bitwidth {bitwidth}: {value!r}")
        data = encode_exact(value.network_address, bitwidth)
        return data, value.prefixlen

    if isinstance(value, int):
        vals = [value]
    elif isinstance(value, str):
        vals = value.split("/", 1)
    elif isinstance(value, tuple):  # pyright: ignore [reportUnnecessaryIsInstance]
        if len(value) != 2:
            raise ValueError(f"invalid tuple value: {value!r}")
        vals = value
    else:
        raise ValueError(f"unexpected type: {value!r}")

    data = encode_exact(vals[0], bitwidth)
    if len(vals) == 2:
        prefix = int(vals[1])
    else:
        prefix = bitwidth

    if prefix > bitwidth or prefix < 0:
        raise ValueError(f"invalid prefix for bitwidth {bitwidth}: {value!r}")

    return data, prefix


def decode_lpm(
    data: bytes,
    prefix_len: int,
    bitwidth: int,
    hint: DecodeHint = DecodeHint.DEFAULT,
) -> _LPMReturn:
    "Decode a P4R Value into an integer or address."

    return (decode_exact(data, bitwidth, hint), prefix_len)


def encode_ternary(value: _TernaryValue, bitwidth: int) -> tuple[bytes, bytes]:
    "Encode a value into a P4R ternary value."
    assert value is not None

    if isinstance(value, (IPv4Network, IPv6Network)):
        val, mask = int(value.network_address), int(value.netmask)
    else:
        match value:
            case str(val):
                val, mask = val.split("/", 1)
            case (val, mask):
                pass
            case other:
                raise ValueError(f"unexpected value: {other!r}")

    return (encode_exact(val, bitwidth), encode_exact(mask, bitwidth))


def decode_ternary(
    data: bytes,
    mask: bytes,
    bitwidth: int,
    hint: DecodeHint = DecodeHint.DEFAULT,
) -> _TernaryReturn:
    "Decode a P4R Ternary value."

    return (decode_exact(data, bitwidth, hint), decode_exact(mask, bitwidth, hint))


def encode_range(value: _RangeValue, bitwidth: int) -> tuple[bytes, bytes]:
    "Encode a P4R Range value."
    assert value is not None

    match value:
        case str(val):
            low, high = val.split("...", 1)
        case (low, high):
            pass
        case other:
            raise ValueError(f"unexpected value: {other!r}")

    return (encode_exact(low, bitwidth), encode_exact(high, bitwidth))


def decode_range(
    low: bytes,
    high: bytes,
    bitwidth: int,
    hint: DecodeHint = DecodeHint.DEFAULT,
) -> _RangeReturn:
    "Decode a P4R Range value."

    return (decode_exact(low, bitwidth, hint), decode_exact(high, bitwidth, hint))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: p4/config/v1/p4types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ap4/config/v1/p4types.proto\x12\x0cp4.config.v1\"\xa1\x07\n\nP4TypeInfo\x12\x36\n\x07structs\x18\x01 \x03(\x0b\x32%.p4.config.v1.P4TypeInfo.StructsEntry\x12\x36\n\x07headers\x18\x02 \x03(\x0b\x32%.p4.config.v1.P4TypeInfo.HeadersEntry\x12\x41\n\rheader_unions\x18\x03 \x03(\x0b\x32*.p4.config.v1.P4TypeInfo.HeaderUnionsEntry\x12\x32\n\x05\x65nums\x18\x04 \x03(\x0b\x32#.p4.config.v1.P4TypeInfo.EnumsEntry\x12,\n\x05\x65rror\x18\x05 \x01(\x0b\x32\x1d.p4.config.v1.P4ErrorTypeSpec\x12K\n\x12serializable_enums\x18\x06 \x03(\x0b\x32/.p4.config.v1.P4TypeInfo.SerializableEnumsEntry\x12\x39\n\tnew_types\x18\x07 \x03(\x0b\x32&.p4.config.v1.P4TypeInfo.NewTypesEntry\x1aN\n\x0cStructsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.p4.config.v1.P4StructTypeSpec:\x02\x38\x01\x1aN\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.p4.config.v1.P4HeaderTypeSpec:\x02\x38\x01\x1aX\n\x11HeaderUnionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.p4.config.v1.P4HeaderUnionTypeSpec:\x02\x38\x01\x1aJ\n\nEnumsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.p4.config.v1.P4EnumTypeSpec:\x02\x38\x01\x1a\x62\n\x16SerializableEnumsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.p4.config.v1.P4SerializableEnumTypeSpec:\x02\x38\x01\x1aL\n\rNewTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x1b.p4.config.v1.P4NewTypeSpec:\x02\x38\x01\"\x83\x05\n\x0eP4DataTypeSpec\x12:\n\tbitstring\x18\x01 \x01(\x0b\x32%.p4.config.v1.P4BitstringLikeTypeSpecH\x00\x12(\n\x04\x62ool\x18\x02 \x01(\x0b\x32\x18.p4.config.v1.P4BoolTypeH\x00\x12.\n\x05tuple\x18\x03 \x01(\x0b\x32\x1d.p4.config.v1.P4TupleTypeSpecH\x00\x12+\n\x06struct\x18\x04 \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x12+\n\x06header\x18\x05 \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x12\x31\n\x0cheader_union\x18\x06 \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x12;\n\x0cheader_stack\x18\x07 \x01(\x0b\x32#.p4.config.v1.P4HeaderStackTypeSpecH\x00\x12\x46\n\x12header_union_stack\x18\x08 \x01(\x0b\x32(.p4.config.v1.P4HeaderUnionStackTypeSpecH\x00\x12)\n\x04\x65num\x18\t \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x12*\n\x05\x65rror\x18\n \x01(\x0b\x32\x19.p4.config.v1.P4ErrorTypeH\x00\x12\x36\n\x11serializable_enum\x18\x0b \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x12-\n\x08new_type\x18\x0c \x01(\x0b\x32\x19.p4.config.v1.P4NamedTypeH\x00\x42\x0b\n\ttype_spec\"\x1b\n\x0bP4NamedType\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x0c\n\nP4BoolType\"\r\n\x0bP4ErrorType\"\xc5\x02\n\x17P4BitstringLikeTypeSpec\x12*\n\x03\x62it\x18\x01 \x01(\x0b\x32\x1b.p4.config.v1.P4BitTypeSpecH\x00\x12*\n\x03int\x18\x02 \x01(\x0b\x32\x1b.p4.config.v1.P4IntTypeSpecH\x00\x12\x30\n\x06varbit\x18\x03 \x01(\x0b\x32\x1e.p4.config.v1.P4VarbitTypeSpecH\x00\x12\x13\n\x0b\x61nnotations\x18\x04 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x05 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x06 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotationB\x0b\n\ttype_spec\"!\n\rP4BitTypeSpec\x12\x10\n\x08\x62itwidth\x18\x01 \x01(\x05\"!\n\rP4IntTypeSpec\x12\x10\n\x08\x62itwidth\x18\x01 \x01(\x05\"(\n\x10P4VarbitTypeSpec\x12\x14\n\x0cmax_bitwidth\x18\x01 \x01(\x05\"@\n\x0fP4TupleTypeSpec\x12-\n\x07members\x18\x01 \x03(\x0b\x32\x1c.p4.config.v1.P4DataTypeSpec\"\xa8\x02\n\x10P4StructTypeSpec\x12\x36\n\x07members\x18\x01 \x03(\x0b\x32%.p4.config.v1.P4StructTypeSpec.Member\x12\x13\n\x0b\x61nnotations\x18\x02 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x03 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\x1aG\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\ttype_spec\x18\x02 \x01(\x0b\x32\x1c.p4.config.v1.P4DataTypeSpec\"\xb1\x02\n\x10P4HeaderTypeSpec\x12\x36\n\x07members\x18\x01 \x03(\x0b\x32%.p4.config.v1.P4HeaderTypeSpec.Member\x12\x13\n\x0b\x61nnotations\x18\x02 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x03 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\x1aP\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x38\n\ttype_spec\x18\x02 \x01(\x0b\x32%.p4.config.v1.P4BitstringLikeTypeSpec\"\xac\x02\n\x15P4HeaderUnionTypeSpec\x12;\n\x07members\x18\x01 \x03(\x0b\x32*.p4.config.v1.P4HeaderUnionTypeSpec.Member\x12\x13\n\x0b\x61nnotations\x18\x02 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x03 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\x1a\x41\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12)\n\x06header\x18\x02 \x01(\x0b\x32\x19.p4.config.v1.P4NamedType\"P\n\x15P4HeaderStackTypeSpec\x12)\n\x06header\x18\x01 \x01(\x0b\x32\x19.p4.config.v1.P4NamedType\x12\x0c\n\x04size\x18\x02 \x01(\x05\"[\n\x1aP4HeaderUnionStackTypeSpec\x12/\n\x0cheader_union\x18\x01 \x01(\x0b\x32\x19.p4.config.v1.P4NamedType\x12\x0c\n\x04size\x18\x02 \x01(\x05\"D\n\x0cKeyValuePair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.p4.config.v1.Expression\"@\n\x10KeyValuePairList\x12,\n\x08kv_pairs\x18\x01 \x03(\x0b\x32\x1a.p4.config.v1.KeyValuePair\"Z\n\nExpression\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x15\n\x0bint64_value\x18\x02 \x01(\x03H\x00\x12\x14\n\nbool_value\x18\x03 \x01(\x08H\x00\x42\x07\n\x05value\"?\n\x0e\x45xpressionList\x12-\n\x0b\x65xpressions\x18\x01 \x03(\x0b\x32\x18.p4.config.v1.Expression\"\xd4\x01\n\x14StructuredAnnotation\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x37\n\x0f\x65xpression_list\x18\x02 \x01(\x0b\x32\x1c.p4.config.v1.ExpressionListH\x00\x12\x36\n\x0ckv_pair_list\x18\x03 \x01(\x0b\x32\x1e.p4.config.v1.KeyValuePairListH\x00\x12\x35\n\x0fsource_location\x18\x04 \x01(\x0b\x32\x1c.p4.config.v1.SourceLocationB\x06\n\x04\x62ody\"<\n\x0eSourceLocation\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\x0c\n\x04line\x18\x02 \x01(\x05\x12\x0e\n\x06\x63olumn\x18\x03 \x01(\x05\"\x89\x03\n\x0eP4EnumTypeSpec\x12\x34\n\x07members\x18\x01 \x03(\x0b\x32#.p4.config.v1.P4EnumTypeSpec.Member\x12\x13\n\x0b\x61nnotations\x18\x02 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x04 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x03 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\x1a\xab\x01\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61nnotations\x18\x02 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x04 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x03 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\"\xe6\x03\n\x1aP4SerializableEnumTypeSpec\x12\x34\n\x0funderlying_type\x18\x01 \x01(\x0b\x32\x1b.p4.config.v1.P4BitTypeSpec\x12@\n\x07members\x18\x02 \x03(\x0b\x32/.p4.config.v1.P4SerializableEnumTypeSpec.Member\x12\x13\n\x0b\x61nnotations\x18\x03 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x05 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\x1a\xba\x01\n\x06Member\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x13\n\x0b\x61nnotations\x18\x03 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x05 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotation\"\"\n\x0fP4ErrorTypeSpec\x12\x0f\n\x07members\x18\x01 \x03(\t\"\x98\x01\n\x14P4NewTypeTranslation\x12\x0b\n\x03uri\x18\x01 \x01(\t\x12\x16\n\x0csdn_bitwidth\x18\x02 \x01(\x05H\x00\x12\x42\n\nsdn_string\x18\x03 \x01(\x0b\x32,.p4.config.v1.P4NewTypeTranslation.SdnStringH\x00\x1a\x0b\n\tSdnStringB\n\n\x08sdn_type\"\xac\x02\n\rP4NewTypeSpec\x12\x35\n\roriginal_type\x18\x01 \x01(\x0b\x32\x1c.p4.config.v1.P4DataTypeSpecH\x00\x12=\n\x0ftranslated_type\x18\x02 \x01(\x0b\x32\".p4.config.v1.P4NewTypeTranslationH\x00\x12\x13\n\x0b\x61nnotations\x18\x03 \x03(\t\x12:\n\x14\x61nnotation_locations\x18\x05 \x03(\x0b\x32\x1c.p4.config.v1.SourceLocation\x12\x42\n\x16structured_annotations\x18\x04 \x03(\x0b\x32\".p4.config.v1.StructuredAnnotationB\x10\n\x0erepresentationB-Z+github.com/p4lang/p4runtime/go/p4/config/v1b\x06proto3')



_P4TYPEINFO = DESCRIPTOR.message_types_by_name['P4TypeInfo']
_P4TYPEINFO_STRUCTSENTRY = _P4TYPEINFO.nested_types_by_name['StructsEntry']
_P4TYPEINFO_HEADERSENTRY = _P4TYPEINFO.nested_types_by_name['HeadersEntry']
_P4TYPEINFO_HEADERUNIONSENTRY = _P4TYPEINFO.nested_types_by_name['HeaderUnionsEntry']
_P4TYPEINFO_ENUMSENTRY = _P4TYPEINFO.nested_types_by_name['EnumsEntry']
_P4TYPEINFO_SERIALIZABLEENUMSENTRY = _P4TYPEINFO.nested_types_by_name['SerializableEnumsEntry']
_P4TYPEINFO_NEWTYPESENTRY = _P4TYPEINFO.nested_types_by_name['NewTypesEntry']
_P4DATATYPESPEC = DESCRIPTOR.message_types_by_name['P4DataTypeSpec']
_P4NAMEDTYPE = DESCRIPTOR.message_types_by_name['P4NamedType']
_P4BOOLTYPE = DESCRIPTOR.message_types_by_name['P4BoolType']
_P4ERRORTYPE = DESCRIPTOR.message_types_by_name['P4ErrorType']
_P4BITSTRINGLIKETYPESPEC = DESCRIPTOR.message_types_by_name['P4BitstringLikeTypeSpec']
_P4BITTYPESPEC = DESCRIPTOR.message_types_by_name['P4BitTypeSpec']
_P4INTTYPESPEC = DESCRIPTOR.message_types_by_name['P4IntTypeSpec']
_P4VARBITTYPESPEC = DESCRIPTOR.message_types_by_name['P4VarbitTypeSpec']
_P4TUPLETYPESPEC = DESCRIPTOR.message_types_by_name['P4TupleTypeSpec']
_P4STRUCTTYPESPEC = DESCRIPTOR.message_types_by_name['P4StructTypeSpec']
_P4STRUCTTYPESPEC_MEMBER = _P4STRUCTTYPESPEC.nested_types_by_name['Member']
_P4HEADERTYPESPEC = DESCRIPTOR.message_types_by_name['P4HeaderTypeSpec']
_P4HEADERTYPESPEC_MEMBER = _P4HEADERTYPESPEC.nested_types_by_name['Member']
_P4HEADERUNIONTYPESPEC = DESCRIPTOR.message_types_by_name['P4HeaderUnionTypeSpec']
_P4HEADERUNIONTYPESPEC_MEMBER = _P4HEADERUNIONTYPESPEC.nested_types_by_name['Member']
_P4HEADERSTACKTYPESPEC = DESCRIPTOR.message_types_by_name['P4HeaderStackTypeSpec']
_P4HEADERUNIONSTACKTYPESPEC = DESCRIPTOR.message_types_by_name['P4HeaderUnionStackTypeSpec']
_KEYVALUEPAIR = DESCRIPTOR.message_types_by_name['KeyValuePair']
_KEYVALUEPAIRLIST = DESCRIPTOR.message_types_by_name['KeyValuePairList']
_EXPRESSION = DESCRIPTOR.message_types_by_name['Expression']
_EXPRESSIONLIST = DESCRIPTOR.message_types_by_name['ExpressionList']
_STRUCTUREDANNOTATION = DESCRIPTOR.message_types_by_name['StructuredAnnotation']
_SOURCELOCATION = DESCRIPTOR.message_types_by_name['SourceLocation']
_P4ENUMTYPESPEC = DESCRIPTOR.message_types_by_name['P4EnumTypeSpec']
_P4ENUMTYPESPEC_MEMBER = _P4ENUMTYPESPEC.nested_types_by_name['Member']
_P4SERIALIZABLEENUMTYPESPEC = DESCRIPTOR.message_types_by_name['P4SerializableEnumTypeSpec']
_P4SERIALIZABLEENUMTYPESPEC_MEMBER = _P4SERIALIZABLEENUMTYPESPEC.nested_types_by_name['Member']
_P4ERRORTYPESPEC = DESCRIPTOR.message_types_by_name['P4ErrorTypeSpec']
_P4NEWTYPETRANSLATION = DESCRIPTOR.message_types_by_name['P4NewTypeTranslation']
_P4NEWTYPETRANSLATION_SDNSTRING = _P4NEWTYPETRANSLATION.nested_types_by_name['SdnString']
_P4NEWTYPESPEC = DESCRIPTOR.message_types_by_name['P4NewTypeSpec']
P4TypeInfo = _reflection.GeneratedProtocolMessageType('P4TypeInfo', (_message.Message,), {

  'StructsEntry' : _reflection.GeneratedProtocolMessageType('StructsEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_STRUCTSENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.StructsEntry)
    })
  ,

  'HeadersEntry' : _reflection.GeneratedProtocolMessageType('HeadersEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_HEADERSENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.HeadersEntry)
    })
  ,

  'HeaderUnionsEntry' : _reflection.GeneratedProtocolMessageType('HeaderUnionsEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_HEADERUNIONSENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.HeaderUnionsEntry)
    })
  ,

  'EnumsEntry' : _reflection.GeneratedProtocolMessageType('EnumsEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_ENUMSENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.EnumsEntry)
    })
  ,

  'SerializableEnumsEntry' : _reflection.GeneratedProtocolMessageType('SerializableEnumsEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_SERIALIZABLEENUMSENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.SerializableEnumsEntry)
    })
  ,

  'NewTypesEntry' : _reflection.GeneratedProtocolMessageType('NewTypesEntry', (_message.Message,), {
    'DESCRIPTOR' : _P4TYPEINFO_NEWTYPESENTRY,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo.NewTypesEntry)
    })
  ,
  'DESCRIPTOR' : _P4TYPEINFO,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4TypeInfo)
  })
_sym_db.RegisterMessage(P4TypeInfo)
_sym_db.RegisterMessage(P4TypeInfo.StructsEntry)
_sym_db.RegisterMessage(P4TypeInfo.HeadersEntry)
_sym_db.RegisterMessage(P4TypeInfo.HeaderUnionsEntry)
_sym_db.RegisterMessage(P4TypeInfo.EnumsEntry)
_sym_db.RegisterMessage(P4TypeInfo.SerializableEnumsEntry)
_sym_db.RegisterMessage(P4TypeInfo.NewTypesEntry)

P4DataTypeSpec = _reflection.GeneratedProtocolMessageType('P4DataTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4DATATYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4DataTypeSpec)
  })
_sym_db.RegisterMessage(P4DataTypeSpec)

P4NamedType = _reflection.GeneratedProtocolMessageType('P4NamedType', (_message.Message,), {
  'DESCRIPTOR' : _P4NAMEDTYPE,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4NamedType)
  })
_sym_db.RegisterMessage(P4NamedType)

P4BoolType = _reflection.GeneratedProtocolMessageType('P4BoolType', (_message.Message,), {
  'DESCRIPTOR' : _P4BOOLTYPE,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4BoolType)
  })
_sym_db.RegisterMessage(P4BoolType)

P4ErrorType = _reflection.GeneratedProtocolMessageType('P4ErrorType', (_message.Message,), {
  'DESCRIPTOR' : _P4ERRORTYPE,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4ErrorType)
  })
_sym_db.RegisterMessage(P4ErrorType)

P4BitstringLikeTypeSpec = _reflection.GeneratedProtocolMessageType('P4BitstringLikeTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4BITSTRINGLIKETYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4BitstringLikeTypeSpec)
  })
_sym_db.RegisterMessage(P4BitstringLikeTypeSpec)

P4BitTypeSpec = _reflection.GeneratedProtocolMessageType('P4BitTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4BITTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4BitTypeSpec)
  })
_sym_db.RegisterMessage(P4BitTypeSpec)

P4IntTypeSpec = _reflection.GeneratedProtocolMessageType('P4IntTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4INTTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4IntTypeSpec)
  })
_sym_db.RegisterMessage(P4IntTypeSpec)

P4VarbitTypeSpec = _reflection.GeneratedProtocolMessageType('P4VarbitTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4VARBITTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4VarbitTypeSpec)
  })
_sym_db.RegisterMessage(P4VarbitTypeSpec)

P4TupleTypeSpec = _reflection.GeneratedProtocolMessageType('P4TupleTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4TUPLETYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4TupleTypeSpec)
  })
_sym_db.RegisterMessage(P4TupleTypeSpec)

P4StructTypeSpec = _reflection.GeneratedProtocolMessageType('P4StructTypeSpec', (_message.Message,), {

  'Member' : _reflection.GeneratedProtocolMessageType('Member', (_message.Message,), {
    'DESCRIPTOR' : _P4STRUCTTYPESPEC_MEMBER,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4StructTypeSpec.Member)
    })
  ,
  'DESCRIPTOR' : _P4STRUCTTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4StructTypeSpec)
  })
_sym_db.RegisterMessage(P4StructTypeSpec)
_sym_db.RegisterMessage(P4StructTypeSpec.Member)

P4HeaderTypeSpec = _reflection.GeneratedProtocolMessageType('P4HeaderTypeSpec', (_message.Message,), {

  'Member' : _reflection.GeneratedProtocolMessageType('Member', (_message.Message,), {
    'DESCRIPTOR' : _P4HEADERTYPESPEC_MEMBER,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderTypeSpec.Member)
    })
  ,
  'DESCRIPTOR' : _P4HEADERTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderTypeSpec)
  })
_sym_db.RegisterMessage(P4HeaderTypeSpec)
_sym_db.RegisterMessage(P4HeaderTypeSpec.Member)

P4HeaderUnionTypeSpec = _reflection.GeneratedProtocolMessageType('P4HeaderUnionTypeSpec', (_message.Message,), {

  'Member' : _reflection.GeneratedProtocolMessageType('Member', (_message.Message,), {
    'DESCRIPTOR' : _P4HEADERUNIONTYPESPEC_MEMBER,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderUnionTypeSpec.Member)
    })
  ,
  'DESCRIPTOR' : _P4HEADERUNIONTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderUnionTypeSpec)
  })
_sym_db.RegisterMessage(P4HeaderUnionTypeSpec)
_sym_db.RegisterMessage(P4HeaderUnionTypeSpec.Member)

P4HeaderStackTypeSpec = _reflection.GeneratedProtocolMessageType('P4HeaderStackTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4HEADERSTACKTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderStackTypeSpec)
  })
_sym_db.RegisterMessage(P4HeaderStackTypeSpec)

P4HeaderUnionStackTypeSpec = _reflection.GeneratedProtocolMessageType('P4HeaderUnionStackTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4HEADERUNIONSTACKTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4HeaderUnionStackTypeSpec)
  })
_sym_db.RegisterMessage(P4HeaderUnionStackTypeSpec)

KeyValuePair = _reflection.GeneratedProtocolMessageType('KeyValuePair', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPAIR,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.KeyValuePair)
  })
_sym_db.RegisterMessage(KeyValuePair)

KeyValuePairList = _reflection.GeneratedProtocolMessageType('KeyValuePairList', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPAIRLIST,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.KeyValuePairList)
  })
_sym_db.RegisterMessage(KeyValuePairList)

Expression = _reflection.GeneratedProtocolMessageType('Expression', (_message.Message,), {
  'DESCRIPTOR' : _EXPRESSION,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.Expression)
  })
_sym_db.RegisterMessage(Expression)

ExpressionList = _reflection.GeneratedProtocolMessageType('ExpressionList', (_message.Message,), {
  'DESCRIPTOR' : _EXPRESSIONLIST,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.ExpressionList)
  })
_sym_db.RegisterMessage(ExpressionList)

StructuredAnnotation = _reflection.GeneratedProtocolMessageType('StructuredAnnotation', (_message.Message,), {
  'DESCRIPTOR' : _STRUCTUREDANNOTATION,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.StructuredAnnotation)
  })
_sym_db.RegisterMessage(StructuredAnnotation)

SourceLocation = _reflection.GeneratedProtocolMessageType('SourceLocation', (_message.Message,), {
  'DESCRIPTOR' : _SOURCELOCATION,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.SourceLocation)
  })
_sym_db.RegisterMessage(SourceLocation)

P4EnumTypeSpec = _reflection.GeneratedProtocolMessageType('P4EnumTypeSpec', (_message.Message,), {

  'Member' : _reflection.GeneratedProtocolMessageType('Member', (_message.Message,), {
    'DESCRIPTOR' : _P4ENUMTYPESPEC_MEMBER,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4EnumTypeSpec.Member)
    })
  ,
  'DESCRIPTOR' : _P4ENUMTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4EnumTypeSpec)
  })
_sym_db.RegisterMessage(P4EnumTypeSpec)
_sym_db.RegisterMessage(P4EnumTypeSpec.Member)

P4SerializableEnumTypeSpec = _reflection.GeneratedProtocolMessageType('P4SerializableEnumTypeSpec', (_message.Message,), {

  'Member' : _reflection.GeneratedProtocolMessageType('Member', (_message.Message,), {
    'DESCRIPTOR' : _P4SERIALIZABLEENUMTYPESPEC_MEMBER,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4SerializableEnumTypeSpec.Member)
    })
  ,
  'DESCRIPTOR' : _P4SERIALIZABLEENUMTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4SerializableEnumTypeSpec)
  })
_sym_db.RegisterMessage(P4SerializableEnumTypeSpec)
_sym_db.RegisterMessage(P4SerializableEnumTypeSpec.Member)

P4ErrorTypeSpec = _reflection.GeneratedProtocolMessageType('P4ErrorTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4ERRORTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4ErrorTypeSpec)
  })
_sym_db.RegisterMessage(P4ErrorTypeSpec)

P4NewTypeTranslation = _reflection.GeneratedProtocolMessageType('P4NewTypeTranslation', (_message.Message,), {

  'SdnString' : _reflection.GeneratedProtocolMessageType('SdnString', (_message.Message,), {
    'DESCRIPTOR' : _P4NEWTYPETRANSLATION_SDNSTRING,
    '__module__' : 'p4.config.v1.p4types_pb2'
    # @@protoc_insertion_point(class_scope:p4.config.v1.P4NewTypeTranslation.SdnString)
    })
  ,
  'DESCRIPTOR' : _P4NEWTYPETRANSLATION,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4NewTypeTranslation)
  })
_sym_db.RegisterMessage(P4NewTypeTranslation)
_sym_db.RegisterMessage(P4NewTypeTranslation.SdnString)

P4NewTypeSpec = _reflection.GeneratedProtocolMessageType('P4NewTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _P4NEWTYPESPEC,
  '__module__' : 'p4.config.v1.p4types_pb2'
  # @@protoc_insertion_point(class_scope:p4.config.v1.P4NewTypeSpec)
  })
_sym_db.RegisterMessage(P4NewTypeSpec)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/p4lang/p4runtime/go/p4/config/v1'
  _P4TYPEINFO_STRUCTSENTRY._options = None
  _P4TYPEINFO_STRUCTSENTRY._serialized_options = b'8\001'
  _P4TYPEINFO_HEADERSENTRY._options = None
  _P4TYPEINFO_HEADERSENTRY._serialized_options = b'8\001'
  _P4TYPEINFO_HEADERUNIONSENTRY._options = None
  _P4TYPEINFO_HEADERUNIONSENTRY._serialized_options = b'8\001'
  _P4TYPEINFO_ENUMSENTRY._options = None
  _P4TYPEINFO_ENUMSENTRY._serialized_options = b'8\001'
  _P4TYPEINFO_SERIALIZABLEENUMSENTRY._options = None
  _P4TYPEINFO_SERIALIZABLEENUMSENTRY._serialized_options = b'8\001'
  _P4TYPEINFO_NEWTYPESENTRY._options = None
  _P4TYPEINFO_NEWTYPESENTRY._serialized_options = b'8\001'
  _P4TYPEINFO._serialized_start=45
  _P4TYPEINFO._serialized_end=974
  _P4TYPEINFO_STRUCTSENTRY._serialized_start=472
  _P4TYPEINFO_STRUCTSENTRY._serialized_end=550
  _P4TYPEINFO_HEADERSENTRY._serialized_start=552
  _P4TYPEINFO_HEADERSENTRY._serialized_end=630
  _P4TYPEINFO_HEADERUNIONSENTRY._serialized_start=632
  _P4TYPEINFO_HEADERUNIONSENTRY._serialized_end=720
  _P4TYPEINFO_ENUMSENTRY._serialized_start=722
  _P4TYPEINFO_ENUMSENTRY._serialized_end=796
  _P4TYPEINFO_SERIALIZABLEENUMSENTRY._serialized_start=798
  _P4TYPEINFO_SERIALIZABLEENUMSENTRY._serialized_end=896
  _P4TYPEINFO_NEWTYPESENTRY._serialized_start=898
  _P4TYPEINFO_NEWTYPESENTRY._serialized_end=974
  _P4DATATYPESPEC._serialized_start=977
  _P4DATATYPESPEC._serialized_end=1620
  _P4NAMEDTYPE._serialized_start=1622
  _P4NAMEDTYPE._serialized_end=1649
  _P4BOOLTYPE._serialized_start=1651
  _P4BOOLTYPE._serialized_end=1663
  _P4ERRORTYPE._serialized_start=1665
  _P4ERRORTYPE._serialized_end=1678
  _P4BITSTRINGLIKETYPESPEC._serialized_start=1681
  _P4BITSTRINGLIKETYPESPEC._serialized_end=2006
  _P4BITTYPESPEC._serialized_start=2008
  _P4BITTYPESPEC._serialized_end=2041
  _P4INTTYPESPEC._serialized_start=2043
  _P4INTTYPESPEC._serialized_end=2076
  _P4VARBITTYPESPEC._serialized_start=2078
  _P4VARBITTYPESPEC._serialized_end=2118
  _P4TUPLETYPESPEC._serialized_start=2120
  _P4TUPLETYPESPEC._serialized_end=2184
  _P4STRUCTTYPESPEC._serialized_start=2187
  _P4STRUCTTYPESPEC._serialized_end=2483
  _P4STRUCTTYPESPEC_MEMBER._serialized_start=2412
  _P4STRUCTTYPESPEC_MEMBER._serialized_end=2483
  _P4HEADERTYPESPEC._serialized_start=2486
  _P4HEADERTYPESPEC._serialized_end=2791
  _P4HEADERTYPESPEC_MEMBER._serialized_start=2711
  _P4HEADERTYPESPEC_MEMBER._serialized_end=2791
  _P4HEADERUNIONTYPESPEC._serialized_start=2794
  _P4HEADERUNIONTYPESPEC._serialized_end=3094
  _P4HEADERUNIONTYPESPEC_MEMBER._serialized_start=3029
  _P4HEADERUNIONTYPESPEC_MEMBER._serialized_end=3094
  _P4HEADERSTACKTYPESPEC._serialized_start=3096
  _P4HEADERSTACKTYPESPEC._serialized_end=3176
  _P4HEADERUNIONSTACKTYPESPEC._serialized_start=3178
  _P4HEADERUNIONSTACKTYPESPEC._serialized_end=3269
  _KEYVALUEPAIR._serialized_start=3271
  _KEYVALUEPAIR._serialized_end=3339
  _KEYVALUEPAIRLIST._serialized_start=3341
  _KEYVALUEPAIRLIST._serialized_end=3405
  _EXPRESSION._serialized_start=3407
  _EXPRESSION._serialized_end=3497
  _EXPRESSIONLIST._serialized_start=3499
  _EXPRESSIONLIST._serialized_end=3562
  _STRUCTUREDANNOTATION._serialized_start=3565
  _STRUCTUREDANNOTATION._serialized_end=3777
  _SOURCELOCATION._serialized_start=3779
  _SOURCELOCATION._serialized_end=3839
  _P4ENUMTYPESPEC._serialized_start=3842
  _P4ENUMTYPESPEC._serialized_end=4235
  _P4ENUMTYPESPEC_MEMBER._serialized_start=4064
  _P4ENUMTYPESPEC_MEMBER._serialized_end=4235
  _P4SERIALIZABLEENUMTYPESPEC._serialized_start=4238
  _P4SERIALIZABLEENUMTYPESPEC._serialized_end=4724
  _P4SERIALIZABLEENUMTYPESPEC_MEMBER._serialized_start=4538
  _P4SERIALIZABLEENUMTYPESPEC_MEMBER._serialized_end=4724
  _P4ERRORTYPESPEC._serialized_start=4726
  _P4ERRORTYPESPEC._serialized_end=4760
  _P4NEWTYPETRANSLATION._serialized_start=4763
  _P4NEWTYPETRANSLATION._serialized_end=4915
  _P4NEWTYPETRANSLATION_SDNSTRING._serialized_start=4892
  _P4NEWTYPETRANSLATION_SDNSTRING._serialized_end=4903
  _P4NEWTYPESPEC._serialized_start=4918
  _P4NEWTYPESPEC._serialized_end=5218
# @@protoc_insertion_point(module_scope)
